unit coolhint;

interface

uses Windows, Classes, Controls, Forms, Messages, Graphics;

type
  TCoolHintWindow = class(THintWindow)
  private
    FRegion: THandle;
    procedure FreeCurrentRegion;
  public
    destructor Destroy; override;
    procedure ActivateHint(Rect: TRect; const AHint: String); override;
    procedure Paint; override;
    procedure CreateParams(var Params: TCreateParams); override;
  end;

implementation

destructor TCoolHintWindow.Destroy;
begin
  FreeCurrentRegion;
  inherited Destroy;
end;

procedure TCoolHintWindow.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style:= Params.Style and not ws_Border;
end;

procedure TCoolHintWindow.FreeCurrentRegion;
begin
  if FRegion <> 0 then
  begin
    SetWindowRgn(Handle, 0, True);
    DeleteObject(FRegion);
    FREgion:= 0;
  end;
end;

procedure TCoolHintWindow.ActivateHint(Rect: TRect; const AHint: String);
begin
  with Rect do
    Right:= Right + Canvas.TextWidth('WWWW');
  BoundsRect:= Rect;
  FreeCurrentRegion;
  with BoundsRect do
    FRegion:= CreateRoundRectRgn(0, 0, Width, Height, Width, Height);
  if FRegion <> 0 then
    SetWindowRgn(Handle, Fregion, True);
  inherited ActivateHint(Rect, AHint);
end;

procedure TCoolHintWindow.Paint;
var R: TRect;
begin
  R:= ClientRect;
  Inc(R.Left, 1);
  Canvas.Font.Color:= clInfoText;
  DrawText(Canvas.Handle, PChar(Caption), Length(Caption), R, DT_NOPREFIX or DT_WORDBREAK or DT_CENTER or DT_VCENTER);
end;

var OldHintClass: THintWindowClass;

function SetNewHintClass(AClass: ThintWindowClass): THintWindowClass;
var DoShowHint: Boolean;
begin
  Result:= HintWindowClass;
  DoShowHint:= Application.ShowHint;
  if DoShowHint then
    Application.ShowHint:= False;
  HintWindowClass:= AClass;
  if DoShowHint then
    Application.ShowHint:= True;
end;

initialization
  OldHintClass:= SetNewHintClass(TCoolHintWindow);

finalization
  SetNewHintClass(OldHintClass);

end.
